"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
function last(tree) {
    if (tree._size === 0)
        return void 0;
    var node = tree._root;
    while (!internals_1.isNone(node._right)) {
        node = node._right;
    }
    return node;
}
exports.last = last;
/**
 * Retrieves the last key in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {([K, V]|undefined)} The key of the last entry in the tree, or undefined if the tree was empty
 */
function lastKey(tree) {
    var node = last(tree);
    return core_1.isDefined(node) ? node.key : void 0;
}
exports.lastKey = lastKey;
/**
 * Retrieves the value of the last entry in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {([K, V]|undefined)} The value of the last entry in the tree, or undefined if the tree was empty
 */
function lastValue(tree) {
    var node = last(tree);
    return core_1.isDefined(node) ? node.value : void 0;
}
exports.lastValue = lastValue;
function iterateFromLast(tree) {
    var path = internals_1.PathNode.NONE;
    var node = tree._root;
    while (!internals_1.isNone(node)) {
        path = internals_1.PathNode.next(node, path, 2 /* RIGHT */);
        node = node._right;
    }
    return new internals_1.RedBlackTreeIterator(path, true);
}
exports.iterateFromLast = iterateFromLast;

//# sourceMappingURL=last.js.map
