import { isDefined } from '@collectable/core';
import { RedBlackTreeIterator, PathNode, isNone } from '../internals';
export function first(tree) {
    if (tree._size === 0)
        return void 0;
    var node = tree._root;
    while (!isNone(node._left)) {
        node = node._left;
    }
    return node;
}
/**
 * Retrieves the first key in the tree, or undefined if the tree is empty.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {(K|undefined)} The first key in the tree, or undefined if the tree is empty
 */
export function firstKey(tree) {
    var node = first(tree);
    return isDefined(node) ? node.key : void 0;
}
/**
 * Retrieves the value of the first entry in the tree, or undefined if the tree is empty.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {(K|undefined)} The value of the first entry in the tree, or undefined if the tree is empty
 */
export function firstValue(tree) {
    var node = first(tree);
    return isDefined(node) ? node.value : void 0;
}
export function iterateFromFirst(tree) {
    var path = PathNode.NONE;
    var node = tree._root;
    while (!isNone(node)) {
        path = PathNode.next(node, path, 1 /* LEFT */);
        node = node._left;
    }
    return new RedBlackTreeIterator(path, false);
}

//# sourceMappingURL=first.js.map
