import { isDefined } from '@collectable/core';
import { RedBlackTreeIterator, PathNode, isNone } from '../internals';
export function last(tree) {
    if (tree._size === 0)
        return void 0;
    var node = tree._root;
    while (!isNone(node._right)) {
        node = node._right;
    }
    return node;
}
/**
 * Retrieves the last key in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {([K, V]|undefined)} The key of the last entry in the tree, or undefined if the tree was empty
 */
export function lastKey(tree) {
    var node = last(tree);
    return isDefined(node) ? node.key : void 0;
}
/**
 * Retrieves the value of the last entry in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {([K, V]|undefined)} The value of the last entry in the tree, or undefined if the tree was empty
 */
export function lastValue(tree) {
    var node = last(tree);
    return isDefined(node) ? node.value : void 0;
}
export function iterateFromLast(tree) {
    var path = PathNode.NONE;
    var node = tree._root;
    while (!isNone(node)) {
        path = PathNode.next(node, path, 2 /* RIGHT */);
        node = node._right;
    }
    return new RedBlackTreeIterator(path, true);
}

//# sourceMappingURL=last.js.map
